'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _vscodeLanguageserver = require('vscode-languageserver');

var _log4js = require('log4js');

var _documentStorage = require('./document-storage');

var _documentStorage2 = _interopRequireDefault(_documentStorage);

var _astHelper = require('./ast-helper');

var _astHelper2 = _interopRequireDefault(_astHelper);

var _fixes = require('./fixes');

var _fixes2 = _interopRequireDefault(_fixes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Server {

  constructor(connection) {
    this.logger = (0, _log4js.getLogger)('server');
    this.connection = connection;
    this.documentStorage = new _documentStorage2.default(connection);

    const fixerOpts = {
      documentStorage: this.documentStorage,
      astHelper: new _astHelper2.default(this.logger)
    };

    this.fixers = _fixes2.default.map(Cls => new Cls(fixerOpts));

    this.logger.debug('Fixers active: ', this.fixers.map(f => f.type).join(', '));

    connection.onShutdown(() => {
      this.logger.debug('LSP server connection shutting down');
    });

    connection.onInitialize(() => this.onInit());

    connection.listen();
  }

  async onInit() {
    this.connection.onCodeAction(p => this.onCodeAction(p));
    this.connection.onExecuteCommand(p => this.onExecuteCommand(p));

    this.logger.debug('LSP server initialized');
    const commands = this.fixers.map(f => f.type);

    return {
      capabilities: {
        codeActionProvider: true,
        executeCommandProvider: {
          commands
        }
      }
    };
  }

  async onCodeAction(params) {
    try {
      const { range, textDocument } = params;
      const { uri } = textDocument;
      const location = { uri, range };

      const commands = [];
      console.log(location);
      for (const fixer of this.fixers) {
        commands.push(...fixer.suggestCommands(location));
      }

      return commands;
    } catch (e) {
      this.logger.error(e);
    }

    return [];
  }

  async onExecuteCommand(params) {
    try {
      const fixer = this.getFixerOrThrow(params.command);
      const edit = fixer.createEdit(params.arguments);
      const res = await this.connection.workspace.applyEdit(edit);

      this.logger.debug(res.applied ? 'edit applied' : 'edit not applied', edit);
    } catch (e) {
      this.logger.error(e);
    }
  }

  getFixerOrThrow(type) {
    const fixer = this.fixers.find(f => f.type === type);

    if (!fixer) {
      throw new Error(`Fixer "${type}" not found`);
    }

    return fixer;
  }
}
exports.default = Server;